/*	Copyright (C) 1996 Free Software Foundation, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, the Free Software Foundation gives permission
 * for additional uses of the text contained in its release of GUILE.
 *
 * The exception is that, if you link the GUILE library with other files
 * to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the GUILE library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the
 * Free Software Foundation under the name GUILE.  If you copy
 * code from other Free Software Foundation releases into a copy of
 * GUILE, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for GUILE, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.  
 */


#include <stdio.h>
#include <string.h>

#include "getopt.h"
#include "libguile.h"



/* Debugger interface */
#define GDB_TYPE SCM

#include <libguile/gdb_interface.h>

GDB_INTERFACE;



#ifndef GUILE_BOOT_CMD
#define GUILE_BOOT_CMD "(primitive-load-path \"ice-9/boot-9.scm\")"
#endif


/* The main program.  */

/* The name this program was invoked by.  */
char *program_name = NULL;

char usage[] = "Usage: %s [-hv] [--help] [--version]\n";

/* Table of long options.  */
struct option long_opts[] =
  {
    {"version", 0, 0, 'v'},
    {"help", 0, 0, 'h'},
    {0, 0, 0, 0}
  };

void
initplugs ()
{
  // your init functions here
}

extern int scm_boot_guile_1_live;

void
scm_prepare_dump ()
{
    scm_boot_guile_1_live = 0;
}

void
scm_init_after_dump ()
{
    scm_init_load_path ();
}

static void
dodump (char *progname, char *dumpfile)
{
  scm_prepare_dump ();
  scm_igc ("dump");
  sbrk (4096);  /* work around a bug somewhere */
  unexec (dumpfile, progname, 0, 0, 0);
  printf ("dumped `%s'.\n", dumpfile);
}

static void
guile_main (void *closure, int argc, char **argv)
{
  static int dumped = 0;

  char *boot_cmd = (char *) closure;
  char *script = 0;
  char *dump_file = 0;
  int optchar;
  int show_version = 0;
  int show_help = 0;
  int opt_error = 0;

  program_name = strrchr (argv[0], '/');
  if (! program_name)
    program_name = argv[0];
  else
    program_name++;

  initplugs ();

  if (argc > 2 && !strcmp (argv[1], "--dump"))
    {
      int i;

      scm_eval_0str (boot_cmd);
      for (i = 3; i < argc; i++)
	{
	  printf ("loading `%s'\n", argv[i]);
	  scm_eval (scm_listify (SCM_CAR (scm_intern0 ("load")),
				 scm_makfrom0str (argv[i]),
				 SCM_UNDEFINED));
	}
      dumped = 1;
      dodump (program_name, argv[2]);
      return;
    }

  while (1)
    {
      optchar = getopt_long (argc, argv, "+hvs:d:", long_opts, (int *) 0);
      if (optchar == EOF)
	break;

      if (optchar == 's') {
	script = optarg;
	break;
      }

      switch (optchar)
	{
	case 'h':
	  show_help = 1;
	  break;
	case 'v':
	  show_version = 1;
	  break;
	case 'd':
	  dump_file = optarg;
	  break;
	default:
	  opt_error = 1;
	  break;
	}
    }

  if (opt_error)
    {
      /* Print error message and exit.  */
      fprintf (stderr, usage, program_name);
      exit (1);
    }

  if (show_version)
    {
      /* Print version number.  */
      fprintf (stderr, "Guile %s\n", GUILE_VERSION);
      fprintf (stderr, "Copyright (c) 1995, 1996 Free Software Foundation\n");
      fprintf (stderr,
"Guile may be distributed under the terms of the GNU General Public Licence;\n"
"certain other uses are permitted as well.  For details, see the `COPYING',\n"
"which is included in the Guile distribution.\n"
"There is no warranty, to the extent permitted by law.\n");
    }

  if (show_help)
    {
      /* Print help info and exit.  */
      fputs ("This is Guile, a Scheme interpreter.\n", stderr);
      fprintf (stderr, usage, program_name);
      fputs ("  -h, --help\t\t\tPrint a summary of the options\n", stderr);
      fputs ("  -v, --version\t\t\tPrint the version number\n", stderr);
      fputs ("Send bug reports to bug-guile@prep.ai.mit.edu.\n", stderr);
    }

  if (show_version || show_help)
    exit (0);

  if (!dumped)
    scm_eval_0str (boot_cmd);
  else
    scm_init_after_dump ();

  if (script)
    {
      scm_set_program_arguments (argc - optind, argv + optind, script);
      scm_eval (scm_listify (SCM_CAR (scm_intern0 ("load")),
			     scm_makfrom0str (script),
			     SCM_UNDEFINED));
    }
  else
    {
      scm_set_program_arguments (argc - optind, argv + optind, argv[0]);
      scm_eval_0str ("(top-repl)");
    }
}

int 
main (argc, argv)
     int argc;
     char **argv;
{
  scm_boot_guile (argc, argv, guile_main, GUILE_BOOT_CMD);
}
